
% WHAT IF...? MIDDLE LAYER

% FIG 5C columns 1 and 2

tic

mf = 182; % The number of mossy fibres that innervate a field, an estimate 
% derived in the mf stats code paper. Assumes a terminal to gr cell 
% divergence ratio of 1:50
bigmf = 3645; % The number of mossy fibres that innervate a rank
ranks = 41; % The number of ranks that supply parallel fibre input to a 
% microzone in the middle
files = 100; % The number of files that cross ranks at right angles

% (1) GENERATE MOSSY FIBRE RATES RECEIVED BY EACH RANK
% For each rank, rates were randomly sampled from a normal distribution (SD
% 12.5 Hz) with a mean that changes from rank to rank. A plot of the means 
% is sinusoidal

rankrates = zeros(bigmf,ranks); % preallocates storage for an M x ranks matrix

t = 0:0.00625:0.25;
freq = 1;
amp = 125;
wave = amp*sin(2*pi*freq*t+0.5*(pi/2));
wave = wave + 175;

m = wave;
sd = 12.5;

for i = 1:ranks
    rankrates(:,i) = sd.*randn(1,bigmf) + m(i); % array of bigmf integers  
    % randomly drawn from a normal distribution with mean m and standard 
    % deviation sd
end

% (2) SAMPLE MF RATES RECEIVED BY A RANK TO DERIVE RATES RECEIVED BY EACH
% FIELD

fieldrates = cell(files,ranks); % preallocates space for a files x ranks cell 
% matrix
for i = 1:files
    for j = 1:ranks
        sample = datasample(rankrates(:,j),mf);
        fieldrates{i,j} = sample; % each iteration of the loop populates a 
        % cell with mf rates randomly sampled from column j of rankrates
    end
end

% (3) ADD COPIES TO RATES RECEIVED BY EACH FIELD

% mfs contribute a variable number of terminals to a field (each field is 
% represented by a cell in fieldrates).

for i = 1:files
    for j = 1:ranks
        a = [];
        hz = fieldrates{i,j};
        for k = 1:length(hz)
            factor = randi(7); % randomly generated number in the range 1-7
            b = hz(k).*ones(1,factor); % makes copies of each value in the 
            % cell at fieldrates{i,j}. The number of copies is equal to
            % factor
            a = [a b];
        end
        fieldrates{i,j} = a;
    end
end

% (4) PLOT THE MEAN AND SD OF MF RATES IN EACH RANK, AND OF MF RATES IN 
% EACH FILE 

opengl software
figure;

subplot(1,10,1)
M = []; S = [];

for j = 1:ranks
    R = []; 
    for i = 1:files
        R = [R,fieldrates{i,j}];
    end
    M = [M,mean(R)];
    S = [S,std(R)];
end

x = 1:ranks;

plot(x,M,'color',[.4 0 0],'LineWidth',1); 
xlabel('ranks'); ylabel(''); title('input'); 
xlim([1 41]); ylim([30 320]);
xticks([10 20 30 40]);
set(gca,'FontSize',11)
hold on
s = patch([x fliplr(x)], [M-S  fliplr(M+S)], [1 .9 .9],'EdgeColor',[.8 .6 .6]);
alpha(s,.3)

xline(21,'color',[.4 .4 .4],'LineWidth',2,'LineStyle',':')

% % % subplot(1,5,2)
% % % M = []; S = [];
% % % 
% % % for i = 1:files
% % %     R = []; 
% % %     for j = 1:ranks
% % %         R = [R,fieldrates{i,j}];
% % %     end
% % %     M = [M,mean(R)];
% % %     S = [S,std(R)];
% % % end
% % % 
% % % x = 1:files;
% % % 
% % % plot(x,M,'color',[.4 0 0],'LineWidth',1); 
% % % xlabel('files'); ylabel('Hz'); title('mf rates: all files'); 
% % % xlim([1 100]); ylim([0 350]);
% % % xticks([20 40 60 80 100]);
% % % set(gca,'FontSize',11)
% % % hold on
% % % s = patch([x fliplr(x)], [M-S  fliplr(M+S)], [1 .9 .9],'EdgeColor',[.8 .6 .6]);
% % % alpha(s,.3)

% (5) SAMPLE MF RATES IN EACH FIELD TO REPRESENT INPUT TO GR CELLS. TAKE 
% THE MEAN OF EACH SAMPLE AND THE TOP SLICE FOR EACH FIELD

grcellmeans = cell(files,ranks); % Preallocates space
activegrcellmeans = cell(files,ranks);
for i = 1:files
    for j = 1:ranks
        samplemeans = zeros(1,8750); % Preallocates space
        for k = 1:8750
            samplemeans(k) = mean(datasample(fieldrates{i,j},4)); 
        end
        grcellmeans{i,j} = samplemeans; % assigns the sample means to a 
        % cell in grcellmeans. Each cell represents a field. This data
        % isn't graphed but stored away just in case
        order = sort(samplemeans); % puts samplemeans in ascending order
        toporder = order(end-43:end); % 1 x 44 array, the top slice of the
        % gr cell sample means
        activegrcellmeans{i,j} = toporder; % assigns the top slice to a 
        % cell in activegrcellmeans. Each cell represents a field. 
    end
    i
end

% (6) PUT GR CELL RATES IN A ROW OF FIELDS INTO A SINGLE CELL TO REPRESENT
% PF RATES 

pfrates = cell(files,1); % creates a 100 x 1 empty cell array

for i = 1:files % for each row...
    e = [];
    for j = 1:ranks % ...and within each row, each column...
        f = activegrcellmeans{i,j};
        e = [e f];
    end
    pfrates{i,1} = e; % populates the cell array. Each cell contains a 1 x 
    % 1,804 array. (1,804 is derived from 44 sample means per field x 41
    % columns.)
end

% (7) DERIVE THE NUMBER AND FIRING RATE OF ACTIVE PF INPUTS TO 16 STELLATE
% CELLS IN EACH FIELD OF THE CENTRE RANK, AND THE MEAN OF RATES RECEIVED BY
% EACH CELL

SCinputs1 = cell(files,16); % files x 16 empty cell array
SCmeans1 = zeros(files,16);
for i = 1:files
    for j = 1:16
        example = datasample(pfrates{i,1},1);
        TB = inputs(); % a function that returns a number between 1 and 
        % 12 representing the number of active inputs to a stellate
        % cell
        if TB == 0
            sample = example;
        else
            sample = datasample(pfrates{i,1},TB);
        end
        SCinputs1{i,j} = sample; % each iteration of the j  
        % loop populates a cell with pf rates received by a stellate cell; 
        % there are 16 samples because there are 16 stellate cells per
        % PC; sample size is randomly variable with the Table B
        % probabilities; the sampled population is pfrates{i,1}.

        muSC = mean(sample); % The mean rate of input to a stellate cell...
        SCmeans1(i,j) = muSC; %...is assigned to index position i,j in 
        % the 100 x 16 matrix SCmeans 
    end
end

% (8) PLOT INPUT RATES TO SCs IN THE CENTRE RANK

% Convert SCinputrates - i.e., input rates to 16 SCs per field in the 
% centre rank - to a row vector
step1 = cell(files,1); % creates a 100 x 1 empty cell array

for i = 1:files % for each row...
    g = [];
    for j = 1:16 % ...and within each row, each column...
        h = SCinputs1{i,j};
        g = [g h]; % ...unpack the data in a row of cells to create a 
        % vector array...
    end
    step1{i,1} = g; % ...and repack it in a single cell, creating a single 
    % column of cells, each containing all the input rates to SCs in the 
    % centre field. The data in each cell is a row vector but they are
    % different lengths, hence at this point still stored as a cell array
end

aa = [];
for i = 1:files
    bb = step1{i,1};
    aa = [aa bb];
end

array1 = aa;

% % % subplot(1,5,3)
% % % x = 1:length(array1);
% % % plot(x,array1,'LineStyle','none','Marker','.','MarkerSize',4,'color',[.4 .4 .6],'LineWidth',.5); 
% % % xlabel('signals'); ylabel('common scale'); title('SC inputs: home'); 
% % % ylim([0 350]); xticks([2000 4000 6000]);
% % % set(gca,'FontSize',11)

% (9) REPEAT STEP (7) FIVE TIMES TO DERIVE THE SAME DATA FOR A SECOND,
% THIRD, FOURTH, FIFTH AND SIXTH SET OF 16 STELLATE CELLS IN EACH FIELD, 
% ALSO IN THE CENTRE RANK

% second set
SCinputs2 = cell(files,16); % files x 16 empty cell array
SCmeans2 = zeros(files,16);
for i = 1:files
    for j = 1:16
        example = datasample(pfrates{i,1},1);
        TB = inputs(); % a function that returns a number between 1 and 
        % 12 representing the number of active inputs to a stellate
        % cell
        if TB == 0
            sample = example;
        else
            sample = datasample(pfrates{i,1},TB);
        end
        SCinputs2{i,j} = sample; % each iteration of the j  
        % loop populates a cell with pf rates received by a stellate cell; 
        % there are 16 samples because there are 16 stellate cells per
        % PC; sample size is randomly variable with the Table B
        % probabilities; the sampled population is pfrates{i,1}.

        muSC = mean(sample); % The mean rate of input to a stellate cell...
        SCmeans2(i,j) = muSC; %...is assigned to index position i,j in 
        % the 100 x 16 matrix SCmeans2 
    end
end

% third set
SCinputs3 = cell(files,16); % files x 16 empty cell array
SCmeans3 = zeros(files,16);
for i = 1:files
    for j = 1:16
        example = datasample(pfrates{i,1},1);
        TB = inputs(); % a function that returns a number between 1 and 
        % 12 representing the number of active inputs to a stellate
        % cell
        if TB == 0
            sample = example;
        else
            sample = datasample(pfrates{i,1},TB);
        end
        SCinputs3{i,j} = sample; % each iteration of the j  
        % loop populates a cell with pf rates received by a stellate cell; 
        % there are 16 samples because there are 16 stellate cells per
        % PC; sample size is randomly variable with the Table B
        % probabilities; the sampled population is pfrates{i,1}.

        muSC = mean(sample); % The mean rate of input to a stellate cell...
        SCmeans3(i,j) = muSC; %...is assigned to index position i,j in 
        % the 100 x 16 matrix SCmeans2 
    end
end

% fourth set
SCinputs4 = cell(files,16); % files x 16 empty cell array
SCmeans4 = zeros(files,16);
for i = 1:files
    for j = 1:16
        example = datasample(pfrates{i,1},1);
        TB = inputs(); % a function that returns a number between 1 and 
        % 12 representing the number of active inputs to a stellate
        % cell
        if TB == 0
            sample = example;
        else
            sample = datasample(pfrates{i,1},TB);
        end
        SCinputs4{i,j} = sample; % each iteration of the j  
        % loop populates a cell with pf rates received by a stellate cell; 
        % there are 16 samples because there are 16 stellate cells per
        % PC; sample size is randomly variable with the Table B
        % probabilities; the sampled population is pfrates{i,1}.

        muSC = mean(sample); % The mean rate of input to a stellate cell...
        SCmeans4(i,j) = muSC; %...is assigned to index position i,j in 
        % the 100 x 16 matrix SCmeans2 
    end
end

% fifth set
SCinputs5 = cell(files,16); % files x 16 empty cell array
SCmeans5 = zeros(files,16);
for i = 1:files
    for j = 1:16
        example = datasample(pfrates{i,1},1);
        TB = inputs(); % a function that returns a number between 1 and 
        % 12 representing the number of active inputs to a stellate
        % cell
        if TB == 0
            sample = example;
        else
            sample = datasample(pfrates{i,1},TB);
        end
        SCinputs5{i,j} = sample; % each iteration of the j  
        % loop populates a cell with pf rates received by a stellate cell; 
        % there are 16 samples because there are 16 stellate cells per
        % PC; sample size is randomly variable with the Table B
        % probabilities; the sampled population is pfrates{i,1}.

        muSC = mean(sample); % The mean rate of input to a stellate cell...
        SCmeans5(i,j) = muSC; %...is assigned to index position i,j in 
        % the 100 x 16 matrix SCmeans2 
    end
end

% sixth set
SCinputs6 = cell(files,16); % files x 16 empty cell array
SCmeans6 = zeros(files,16);
for i = 1:files
    for j = 1:16
        example = datasample(pfrates{i,1},1);
        TB = inputs(); % a function that returns a number between 1 and 
        % 12 representing the number of active inputs to a stellate
        % cell
        if TB == 0
            sample = example;
        else
            sample = datasample(pfrates{i,1},TB);
        end
        SCinputs6{i,j} = sample; % each iteration of the j  
        % loop populates a cell with pf rates received by a stellate cell; 
        % there are 16 samples because there are 16 stellate cells per
        % PC; sample size is randomly variable with the Table B
        % probabilities; the sampled population is pfrates{i,1}.

        muSC = mean(sample); % The mean rate of input to a stellate cell...
        SCmeans6(i,j) = muSC; %...is assigned to index position i,j in 
        % the 100 x 16 matrix SCmeans2 
    end
end

% (10) ADD LATERALLY POSITIONED SCs TO INHIBITION OF PCs

L = 4; % The number of lateral SC inputs to a PC from a neighbouring field

% Add lateral inhibition to the first set of stellate cell data
extras = zeros(files,2.*L);
A = datasample(SCmeans1(100,:),L);
B = datasample(SCmeans1(2,:),L);
C = [A B]; 
extras(1,:) = C;
for i = 2:files - 1
    A = datasample(SCmeans1(i - 1,:),L);
    B = datasample(SCmeans1(i + 1,:),L);
    C = [A B];
    extras(i,:) = C;
end
A = datasample(SCmeans1(99,:),L);
B = datasample(SCmeans1(1,:),L);
C = [A B]; 
extras(100,:) = C;
SCmeans1 = [SCmeans1 extras];

% (11) REPEAT STEP 10 FIVE TIMES TO ADD LATERAL INHIBITION TO THE OTHER SETS
% OF STELLATE CELLS

% Add lateral inhibition to the second set of stellate cell data
extras = zeros(files,2.*L);
A = datasample(SCmeans2(100,:),L);
B = datasample(SCmeans2(2,:),L);
C = [A B]; 
extras(1,:) = C;
for i = 2:files - 1
    A = datasample(SCmeans2(i - 1,:),L);
    B = datasample(SCmeans2(i + 1,:),L);
    C = [A B];
    extras(i,:) = C;
end
A = datasample(SCmeans2(99,:),L);
B = datasample(SCmeans2(1,:),L);
C = [A B]; 
extras(100,:) = C;
SCmeans2 = [SCmeans2 extras];

% Add lateral inhibition to the third set of stellate cell data
extras = zeros(files,2.*L);
A = datasample(SCmeans3(100,:),L);
B = datasample(SCmeans3(2,:),L);
C = [A B]; 
extras(1,:) = C;
for i = 2:files - 1
    A = datasample(SCmeans3(i - 1,:),L);
    B = datasample(SCmeans3(i + 1,:),L);
    C = [A B];
    extras(i,:) = C;
end
A = datasample(SCmeans3(99,:),L);
B = datasample(SCmeans3(1,:),L);
C = [A B]; 
extras(100,:) = C;
SCmeans3 = [SCmeans3 extras];

% Add lateral inhibition to the fourth set of stellate cell data
extras = zeros(files,2.*L);
A = datasample(SCmeans4(100,:),L);
B = datasample(SCmeans4(2,:),L);
C = [A B]; 
extras(1,:) = C;
for i = 2:files - 1
    A = datasample(SCmeans4(i - 1,:),L);
    B = datasample(SCmeans4(i + 1,:),L);
    C = [A B];
    extras(i,:) = C;
end
A = datasample(SCmeans4(99,:),L);
B = datasample(SCmeans4(1,:),L);
C = [A B]; 
extras(100,:) = C;
SCmeans4 = [SCmeans4 extras];

% Add lateral inhibition to the fifth set of stellate cell data
extras = zeros(files,2.*L);
A = datasample(SCmeans5(100,:),L);
B = datasample(SCmeans5(2,:),L);
C = [A B]; 
extras(1,:) = C;
for i = 2:files - 1
    A = datasample(SCmeans5(i - 1,:),L);
    B = datasample(SCmeans5(i + 1,:),L);
    C = [A B];
    extras(i,:) = C;
end
A = datasample(SCmeans5(99,:),L);
B = datasample(SCmeans5(1,:),L);
C = [A B]; 
extras(100,:) = C;
SCmeans5 = [SCmeans5 extras];

% Add lateral inhibition to the sixth set of stellate cell data
extras = zeros(files,2.*L);
A = datasample(SCmeans6(100,:),L);
B = datasample(SCmeans6(2,:),L);
C = [A B]; 
extras(1,:) = C;
for i = 2:files - 1
    A = datasample(SCmeans6(i - 1,:),L);
    B = datasample(SCmeans6(i + 1,:),L);
    C = [A B];
    extras(i,:) = C;
end
A = datasample(SCmeans6(99,:),L);
B = datasample(SCmeans6(1,:),L);
C = [A B]; 
extras(100,:) = C;
SCmeans6 = [SCmeans6 extras];

% (12) IN EACH FIELD, SAMPLE SC RATES TO REPRESENT INHIBITION OF PC 
% DENDRITIC SECTORS, AND TAKE THE MEAN OF INHIBITION OF EACH SECTOR

LL = 5; % The number of convergent SC inputs to a PC sector
S = 16; % The number of sectors per PC

% PC1
% Sample the first set of stellate cells
PC1sectorsA = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans1(i,:),LL);
        sectormean = mean(sectorsample);
        PC1sectorsA(i,j) = sectormean;
    end
end

% Sample the second set of stellate cells
PC1sectorsB = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans2(i,:),LL);
        sectormean = mean(sectorsample);
        PC1sectorsB(i,j) = sectormean;
    end
end

PC1sectormeans = zeros(files,S);
for i = 1:files
    for j = 1:S
        D = PC1sectorsA(i,j);
        E = PC1sectorsB(i,j);
        F = mean([D E]);
        PC1sectormeans(i,j) = F;
    end
end

% (13) REPEAT 4 TIMES REPRESENTING THE OTHER FOUR PCs IN A FIELD

% PC2
% Sample the second set of stellate cells
PC2sectorsA = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans2(i,:),LL);
        sectormean = mean(sectorsample);
        PC2sectorsA(i,j) = sectormean;
    end
end

% Sample the third set of stellate cells
PC2sectorsB = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans3(i,:),LL);
        sectormean = mean(sectorsample);
        PC2sectorsB(i,j) = sectormean;
    end
end

PC2sectormeans = zeros(files,S);
for i = 1:files
    for j = 1:S
        D = PC2sectorsA(i,j);
        E = PC2sectorsB(i,j);
        F = mean([D E]);
        PC2sectormeans(i,j) = F;
    end
end

% PC3
% Sample the third set of stellate cells
PC3sectorsA = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans3(i,:),LL);
        sectormean = mean(sectorsample);
        PC3sectorsA(i,j) = sectormean;
    end
end

% Sample the fourth set of stellate cells
PC3sectorsB = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans4(i,:),LL);
        sectormean = mean(sectorsample);
        PC3sectorsB(i,j) = sectormean;
    end
end

PC3sectormeans = zeros(files,S);
for i = 1:files
    for j = 1:S
        D = PC3sectorsA(i,j);
        E = PC3sectorsB(i,j);
        F = mean([D E]);
        PC3sectormeans(i,j) = F;
    end
end

% PC4
% Sample the fourth set of stellate cells
PC4sectorsA = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans4(i,:),LL);
        sectormean = mean(sectorsample);
        PC4sectorsA(i,j) = sectormean;
    end
end

% Sample the fifth set of stellate cells
PC4sectorsB = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans5(i,:),LL);
        sectormean = mean(sectorsample);
        PC4sectorsB(i,j) = sectormean;
    end
end

PC4sectormeans = zeros(files,S);
for i = 1:files
    for j = 1:S
        D = PC4sectorsA(i,j);
        E = PC4sectorsB(i,j);
        F = mean([D E]);
        PC4sectormeans(i,j) = F;
    end
end

% PC5
% Sample the fifth set of stellate cells
PC5sectorsA = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans5(i,:),LL);
        sectormean = mean(sectorsample);
        PC5sectorsA(i,j) = sectormean;
    end
end

% Sample the sixth set of stellate cells
PC5sectorsB = zeros(files,S);
for i = 1:files
    for j = 1:S
        sectorsample = datasample(SCmeans6(i,:),LL);
        sectormean = mean(sectorsample);
        PC5sectorsB(i,j) = sectormean;
    end
end

PC5sectormeans = zeros(files,S);
for i = 1:files
    for j = 1:S
        D = PC5sectorsA(i,j);
        E = PC5sectorsB(i,j);
        F = mean([D E]);
        PC5sectormeans(i,j) = F;
    end
end

% (14) TAKE THE MEAN OF THE SECTOR MEANS FOR EACH PC, REPRESENTING THE
% INTEGRATED EFFECT ON PC SOMATIC MEMBRANE POTENTIAL. 

% Convert PC1sectormeans to a PC1means, 1 x 100 array
flip = PC1sectormeans'; 
PC1means = mean(flip);

% Repeat
flip = PC2sectormeans'; 
PC2means = mean(flip);

flip = PC3sectormeans'; 
PC3means = mean(flip);

flip = PC4sectormeans'; 
PC4means = mean(flip);

flip = PC5sectormeans'; 
PC5means = mean(flip);

% (15) PLOT PC DATA FOR THE CENTRE RANK

PCrankmeans = [PC1means; PC2means; PC3means; PC4means];
PCrankmeans = PCrankmeans(:)';

subplot(1,10,2)
x = 1:length(PCrankmeans);
plot(x,PCrankmeans,'LineStyle','none','Marker','o','MarkerSize',1.5,'Color',[0 .4 0],'MarkerFaceColor',[.64 .83 .57]); 
xlabel('PCs'); ylabel(''); title('output'); 
xlim([0 400]); xticks([0 100 200 300 400]); xticklabels([0 100 200 300 400]);
ylim([30 320]); yticklabels([]);
set(gca,'FontSize',11)

% (16) TAKE 50 RANDOM SAMPLES OF PC DATA (BECAUSE THERE ARE 50 NUCLEAR 
% CELLS) AND TAKE AND PLOT THE SAMPLE MEANS

nuclearmeans = zeros(1,50);

for i = 1:50
    num = randi([30,50]);
    nuclearsample = datasample(PCrankmeans,num);
    nuclearsamplemean = mean(nuclearsample);
    nuclearmeans(1,i) = nuclearsamplemean;
end

toc
